"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EnergyTransferRequest = exports.UpdateStatusCode = exports.TransferValidationStatus = void 0;
var TransferValidationStatus;
(function (TransferValidationStatus) {
    TransferValidationStatus["Pending"] = "pending";
    TransferValidationStatus["Valid"] = "valid";
    TransferValidationStatus["Invalid"] = "invalid";
    TransferValidationStatus["Error"] = "error";
})(TransferValidationStatus = exports.TransferValidationStatus || (exports.TransferValidationStatus = {}));
var UpdateStatusCode;
(function (UpdateStatusCode) {
    UpdateStatusCode["Success"] = "success";
    UpdateStatusCode["NoValidator"] = "noValidator";
    UpdateStatusCode["NotPending"] = "notPending";
})(UpdateStatusCode = exports.UpdateStatusCode || (exports.UpdateStatusCode = {}));
class EnergyTransferRequest {
    constructor(attrs) {
        this.attrs = attrs;
    }
    get certificateId() {
        return this.attrs.certificateId;
    }
    get id() {
        return this.attrs.id;
    }
    get sites() {
        return {
            buyerAddress: this.attrs.buyerAddress,
            sellerAddress: this.attrs.sellerAddress
        };
    }
    get volume() {
        return this.attrs.volume;
    }
    isValid() {
        return Object.values(this.attrs.validationStatusRecord).every((status) => status === TransferValidationStatus.Valid);
    }
    updateCertificateId(certificateId) {
        this.attrs.certificateId = certificateId;
    }
    markCertificatePersisted() {
        this.attrs.isCertificatePersisted = true;
    }
    startValidation(validatorNames) {
        if (validatorNames.length === 0) {
            this.attrs.computedValidationStatus = TransferValidationStatus.Valid;
            return;
        }
        this.attrs.validationStatusRecord = validatorNames.reduce((status, command) => ({
            ...status,
            [command]: TransferValidationStatus.Pending
        }), {});
    }
    updateValidationStatus(validatorName, status) {
        const currentStatus = this.attrs.validationStatusRecord[validatorName];
        if (!currentStatus) {
            return UpdateStatusCode.NoValidator;
        }
        if (currentStatus !== TransferValidationStatus.Pending) {
            return UpdateStatusCode.NotPending;
        }
        this.attrs.validationStatusRecord[validatorName] = status;
        const statuses = Object.values(this.attrs.validationStatusRecord);
        if (statuses.some((s) => s === TransferValidationStatus.Error)) {
            this.attrs.computedValidationStatus = TransferValidationStatus.Error;
        }
        else if (statuses.some((s) => s === TransferValidationStatus.Invalid)) {
            this.attrs.computedValidationStatus = TransferValidationStatus.Invalid;
        }
        else if (statuses.some((s) => s === TransferValidationStatus.Pending)) {
            this.attrs.computedValidationStatus = TransferValidationStatus.Pending;
        }
        else if (statuses.every((s) => s === TransferValidationStatus.Valid)) {
            this.attrs.computedValidationStatus = TransferValidationStatus.Valid;
        }
        return UpdateStatusCode.Success;
    }
    toAttrs() {
        return this.attrs;
    }
    toPublicAttrs() {
        return {
            buyerAddress: this.attrs.buyerAddress,
            sellerAddress: this.attrs.sellerAddress,
            generatorId: this.attrs.generatorId,
            id: this.attrs.id,
            transferDate: this.attrs.transferDate,
            volume: this.attrs.volume
        };
    }
    static fromAttrs(attrs) {
        return new EnergyTransferRequest(attrs);
    }
    static newAttributes(params) {
        return {
            ...params,
            createdAt: new Date(),
            updatedAt: new Date(),
            certificateId: null,
            isCertificatePersisted: false,
            validationStatusRecord: {},
            computedValidationStatus: TransferValidationStatus.Pending
        };
    }
}
exports.EnergyTransferRequest = EnergyTransferRequest;
//# sourceMappingURL=EnergyTransferRequest.js.map